@@ -1,5 +1,9 @@
 #!/bin/bash
 
+# currently not used
+# script to make sure that cache is clean
+# Travis CI now handles this
+
 if [ "$TRAVIS_PULL_REQUEST" == "false" ]
 then
     echo "Not a PR: checking for changes in ci/ from last 2 commits"
@@ -12,14 +16,12 @@ else
     ci_changes=$(git diff PR_HEAD~2 --numstat | grep -E "ci/"| wc -l)
 fi
 
-MINICONDA_DIR="$HOME/miniconda/"
 CACHE_DIR="$HOME/.cache/"
 CCACHE_DIR="$HOME/.ccache/"
 
 if [ $ci_changes -ne 0 ]
 then
     echo "Files have changed in ci/ deleting all caches"
-    rm -rf "$MINICONDA_DIR"
     rm -rf "$CACHE_DIR"
     rm -rf "$CCACHE_DIR"
-fi
\ No newline at end of file
+fi
