@@ -41,7 +41,6 @@ import pandas.core.config_init
 
 from pandas.core.api import *
 from pandas.core.sparse.api import *
-from pandas.stats.api import *
 from pandas.tseries.api import *
 from pandas.core.computation.api import *
 from pandas.core.reshape.api import *
@@ -51,7 +50,7 @@ import pandas.tools.plotting
 plot_params = pandas.plotting._style._Options(deprecated=True)
 # do not import deprecate to top namespace
 scatter_matrix = pandas.util._decorators.deprecate(
-    'pandas.scatter_matrix', pandas.plotting.scatter_matrix,
+    'pandas.scatter_matrix', pandas.plotting.scatter_matrix, '0.20.0',
     'pandas.plotting.scatter_matrix')
 
 from pandas.util._print_versions import show_versions
@@ -104,25 +103,25 @@ Main Features
 Here are just a few of the things that pandas does well:
 
   - Easy handling of missing data in floating point as well as non-floating
-    point data
+    point data.
   - Size mutability: columns can be inserted and deleted from DataFrame and
     higher dimensional objects
-  - Automatic and explicit data alignment: objects can  be explicitly aligned
+  - Automatic and explicit data alignment: objects can be explicitly aligned
     to a set of labels, or the user can simply ignore the labels and let
     `Series`, `DataFrame`, etc. automatically align the data for you in
-    computations
+    computations.
   - Powerful, flexible group by functionality to perform split-apply-combine
-    operations on data sets, for both aggregating and transforming data
+    operations on data sets, for both aggregating and transforming data.
   - Make it easy to convert ragged, differently-indexed data in other Python
-    and NumPy data structures into DataFrame objects
+    and NumPy data structures into DataFrame objects.
   - Intelligent label-based slicing, fancy indexing, and subsetting of large
-    data sets
-  - Intuitive merging and joining data sets
-  - Flexible reshaping and pivoting of data sets
-  - Hierarchical labeling of axes (possible to have multiple labels per tick)
+    data sets.
+  - Intuitive merging and joining data sets.
+  - Flexible reshaping and pivoting of data sets.
+  - Hierarchical labeling of axes (possible to have multiple labels per tick).
   - Robust IO tools for loading data from flat files (CSV and delimited),
     Excel files, databases, and saving/loading data from the ultrafast HDF5
-    format
+    format.
   - Time series-specific functionality: date range generation and frequency
     conversion, moving window statistics, moving window linear regressions,
     date shifting and lagging, etc.
