@@ -4,11 +4,13 @@ import pytest
 import numpy as np
 from pandas.compat import zip
 
-from pandas import (Series, Index, isnull,
-                    to_datetime, DatetimeIndex, Timestamp,
-                    Interval, IntervalIndex, Categorical,
-                    cut, qcut, date_range)
+import pandas as pd
+from pandas import (DataFrame, Series, isna, to_datetime, DatetimeIndex, Index,
+                    Timestamp, Interval, IntervalIndex, Categorical,
+                    cut, qcut, date_range, NaT, TimedeltaIndex)
+from pandas.tseries.offsets import Nano, Day
 import pandas.util.testing as tm
+from pandas.api.types import CategoricalDtype as CDT
 
 from pandas.core.algorithms import quantile
 import pandas.core.reshape.tile as tmod
@@ -28,7 +30,8 @@ class TestCut(object):
         result, bins = cut(data, 3, retbins=True)
 
         intervals = IntervalIndex.from_breaks(bins.round(3))
-        expected = intervals.take([0, 0, 0, 1, 2, 0]).astype('category')
+        intervals = intervals.take([0, 0, 0, 1, 2, 0])
+        expected = Categorical(intervals, ordered=True)
         tm.assert_categorical_equal(result, expected)
         tm.assert_almost_equal(bins, np.array([0.1905, 3.36666667,
                                                6.53333333, 9.7]))
@@ -37,7 +40,8 @@ class TestCut(object):
         data = np.array([.2, 1.4, 2.5, 6.2, 9.7, 2.1, 2.575])
         result, bins = cut(data, 4, right=True, retbins=True)
         intervals = IntervalIndex.from_breaks(bins.round(3))
-        expected = intervals.astype('category').take([0, 0, 0, 2, 3, 0, 0])
+        expected = Categorical(intervals, ordered=True)
+        expected = expected.take([0, 0, 0, 2, 3, 0, 0])
         tm.assert_categorical_equal(result, expected)
         tm.assert_almost_equal(bins, np.array([0.1905, 2.575, 4.95,
                                                7.325, 9.7]))
@@ -46,7 +50,8 @@ class TestCut(object):
         data = np.array([.2, 1.4, 2.5, 6.2, 9.7, 2.1, 2.575])
         result, bins = cut(data, 4, right=False, retbins=True)
         intervals = IntervalIndex.from_breaks(bins.round(3), closed='left')
-        expected = intervals.take([0, 0, 0, 2, 3, 0, 1]).astype('category')
+        intervals = intervals.take([0, 0, 0, 2, 3, 0, 1])
+        expected = Categorical(intervals, ordered=True)
         tm.assert_categorical_equal(result, expected)
         tm.assert_almost_equal(bins, np.array([0.2, 2.575, 4.95,
                                                7.325, 9.7095]))
@@ -55,7 +60,8 @@ class TestCut(object):
         data = [.2, 1.4, 2.5, 6.2, 9.7, 2.1]
         result, bins = cut(data, 3, retbins=True)
         intervals = IntervalIndex.from_breaks(bins.round(3))
-        expected = intervals.take([0, 0, 0, 1, 2, 0]).astype('category')
+        intervals = intervals.take([0, 0, 0, 1, 2, 0])
+        expected = Categorical(intervals, ordered=True)
         tm.assert_categorical_equal(result, expected)
         tm.assert_almost_equal(bins, np.array([0.1905, 3.36666667,
                                                6.53333333, 9.7]))
@@ -99,6 +105,12 @@ class TestCut(object):
 
         pytest.raises(ValueError, cut, [1, 2, 3], 0.5)
 
+    @pytest.mark.parametrize('arg', [2, np.eye(2), DataFrame(np.eye(2))])
+    @pytest.mark.parametrize('cut_func', [cut, qcut])
+    def test_cut_not_1d_arg(self, arg, cut_func):
+        with pytest.raises(ValueError):
+            cut_func(arg, 2)
+
     def test_cut_out_of_range_more(self):
         # #1511
         s = Series([0, -1, 0, 1, -3], name='x')
@@ -140,12 +152,12 @@ class TestCut(object):
 
         result_arr = np.asarray(result)
 
-        ex_arr = np.where(isnull(arr), np.nan, result_arr)
+        ex_arr = np.where(isna(arr), np.nan, result_arr)
 
         tm.assert_almost_equal(result_arr, ex_arr)
 
         result = cut(arr, 4, labels=False)
-        ex_result = np.where(isnull(arr), np.nan, result)
+        ex_result = np.where(isna(arr), np.nan, result)
         tm.assert_almost_equal(result, ex_result)
 
     def test_inf_handling(self):
@@ -200,7 +212,7 @@ class TestCut(object):
 
         result = cut(arr, [-1, 0, 1])
 
-        mask = isnull(result)
+        mask = isna(result)
         ex_mask = (arr < -1) | (arr > 1)
         tm.assert_numpy_array_equal(mask, ex_mask)
 
@@ -232,7 +244,7 @@ class TestCut(object):
 
         ii = qcut(values, 4)
 
-        ex_levels = IntervalIndex.from_intervals(
+        ex_levels = IntervalIndex(
             [Interval(-0.001, 2.25),
              Interval(2.25, 4.5),
              Interval(4.5, 6.75),
@@ -244,12 +256,12 @@ class TestCut(object):
         arr[:20] = np.nan
 
         result = qcut(arr, 4)
-        assert isnull(result[:20]).all()
+        assert isna(result[:20]).all()
 
     def test_qcut_index(self):
         result = qcut([0, 2], 2)
-        expected = Index([Interval(-0.001, 1), Interval(1, 2)]).astype(
-            'category')
+        intervals = [Interval(-0.001, 1), Interval(1, 2)]
+        expected = Categorical(intervals, ordered=True)
         tm.assert_categorical_equal(result, expected)
 
     def test_round_frac(self):
@@ -270,10 +282,10 @@ class TestCut(object):
         result = tmod._round_frac(0.000123456, precision=2)
         assert result == 0.00012
 
-    def test_qcut_binning_issues(self):
+    def test_qcut_binning_issues(self, datapath):
         # #1978, 1979
-        path = os.path.join(tm.get_data_path(), 'cut_data.csv')
-        arr = np.loadtxt(path)
+        cut_file = datapath(os.path.join('reshape', 'data', 'cut_data.csv'))
+        arr = np.loadtxt(cut_file)
 
         result = qcut(arr, 20)
 
@@ -299,7 +311,7 @@ class TestCut(object):
         exp_bins = np.linspace(0, 8, num=4).round(3)
         exp_bins[0] -= 0.008
         exp = Series(IntervalIndex.from_breaks(exp_bins, closed='right').take(
-            [0, 0, 0, 1, 1, 1, 2, 2, 2])).astype('category', ordered=True)
+            [0, 0, 0, 1, 1, 1, 2, 2, 2])).astype(CDT(ordered=True))
         tm.assert_series_equal(res, exp)
 
     def test_qcut_return_intervals(self):
@@ -308,7 +320,7 @@ class TestCut(object):
         exp_levels = np.array([Interval(-0.001, 2.664),
                                Interval(2.664, 5.328), Interval(5.328, 8)])
         exp = Series(exp_levels.take([0, 0, 0, 1, 1, 1, 2, 2, 2])).astype(
-            'category', ordered=True)
+            CDT(ordered=True))
         tm.assert_series_equal(res, exp)
 
     def test_series_retbins(self):
@@ -316,21 +328,35 @@ class TestCut(object):
         s = Series(np.arange(4))
         result, bins = cut(s, 2, retbins=True)
         expected = Series(IntervalIndex.from_breaks(
-            [-0.003, 1.5, 3], closed='right').repeat(2)).astype('category',
-                                                                ordered=True)
+            [-0.003, 1.5, 3], closed='right').repeat(2)).astype(
+            CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         result, bins = qcut(s, 2, retbins=True)
         expected = Series(IntervalIndex.from_breaks(
-            [-0.001, 1.5, 3], closed='right').repeat(2)).astype('category',
-                                                                ordered=True)
+            [-0.001, 1.5, 3], closed='right').repeat(2)).astype(
+            CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
+    def test_cut_duplicates_bin(self):
+        # issue 20947
+        values = Series(np.array([1, 3, 5, 7, 9]),
+                        index=["a", "b", "c", "d", "e"])
+        bins = [0, 2, 4, 6, 10, 10]
+        result = cut(values, bins, duplicates='drop')
+        expected = cut(values, pd.unique(bins))
+        tm.assert_series_equal(result, expected)
+
+        pytest.raises(ValueError, cut, values, bins)
+        pytest.raises(ValueError, cut, values, bins, duplicates='raise')
+
+        # invalid
+        pytest.raises(ValueError, cut, values, bins, duplicates='foo')
+
     def test_qcut_duplicates_bin(self):
         # GH 7751
         values = [0, 0, 0, 0, 1, 2, 3]
-        expected = IntervalIndex.from_intervals([Interval(-0.001, 1),
-                                                 Interval(1, 3)])
+        expected = IntervalIndex([Interval(-0.001, 1), Interval(1, 3)])
 
         result = qcut(values, 3, duplicates='drop')
         tm.assert_index_equal(result.categories, expected)
@@ -351,7 +377,7 @@ class TestCut(object):
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(8.999, 9.0),
                                    Interval(8.999, 9.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         s = Series([-9., -9.])
@@ -361,7 +387,7 @@ class TestCut(object):
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(-9.001, -9.0),
                                    Interval(-9.001, -9.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         s = Series([0., 0.])
@@ -371,7 +397,7 @@ class TestCut(object):
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(-0.001, 0.0),
                                    Interval(-0.001, 0.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         s = Series([9])
@@ -380,7 +406,7 @@ class TestCut(object):
         tm.assert_series_equal(result, expected)
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(8.999, 9.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         s = Series([-9])
@@ -389,7 +415,7 @@ class TestCut(object):
         tm.assert_series_equal(result, expected)
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(-9.001, -9.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
         s = Series([0])
@@ -398,7 +424,7 @@ class TestCut(object):
         tm.assert_series_equal(result, expected)
         result = qcut(s, 1)
         intervals = IntervalIndex([Interval(-0.001, 0.0)], closed='right')
-        expected = Series(intervals).astype('category', ordered=True)
+        expected = Series(intervals).astype(CDT(ordered=True))
         tm.assert_series_equal(result, expected)
 
     def test_single_bin(self):
@@ -436,6 +462,37 @@ class TestCut(object):
         result = cut(s, 1, labels=False)
         tm.assert_series_equal(result, expected)
 
+    @pytest.mark.parametrize(
+        "array_1_writeable, array_2_writeable",
+        [(True, True), (True, False), (False, False)])
+    def test_cut_read_only(self, array_1_writeable, array_2_writeable):
+        # issue 18773
+        array_1 = np.arange(0, 100, 10)
+        array_1.flags.writeable = array_1_writeable
+
+        array_2 = np.arange(0, 100, 10)
+        array_2.flags.writeable = array_2_writeable
+
+        hundred_elements = np.arange(100)
+
+        tm.assert_categorical_equal(cut(hundred_elements, array_1),
+                                    cut(hundred_elements, array_2))
+
+
+class TestDatelike(object):
+
+    @pytest.mark.parametrize('s', [
+        Series(DatetimeIndex(['20180101', NaT, '20180103'])),
+        Series(TimedeltaIndex(['0 days', NaT, '2 days']))],
+        ids=lambda x: str(x.dtype))
+    def test_qcut_nat(self, s):
+        # GH 19768
+        intervals = IntervalIndex.from_tuples(
+            [(s[0] - Nano(), s[2] - Day()), np.nan, (s[2] - Day(), s[2])])
+        expected = Series(Categorical(intervals, ordered=True))
+        result = qcut(s, 2)
+        tm.assert_series_equal(result, expected)
+
     def test_datetime_cut(self):
         # GH 14714
         # testing for time data to be present as series
@@ -443,14 +500,14 @@ class TestCut(object):
 
         result, bins = cut(data, 3, retbins=True)
         expected = (
-            Series(IntervalIndex.from_intervals([
+            Series(IntervalIndex([
                 Interval(Timestamp('2012-12-31 23:57:07.200000'),
                          Timestamp('2013-01-01 16:00:00')),
                 Interval(Timestamp('2013-01-01 16:00:00'),
                          Timestamp('2013-01-02 08:00:00')),
                 Interval(Timestamp('2013-01-02 08:00:00'),
                          Timestamp('2013-01-03 00:00:00'))]))
-            .astype('category', ordered=True))
+            .astype(CDT(ordered=True)))
 
         tm.assert_series_equal(result, expected)
 
@@ -472,14 +529,55 @@ class TestCut(object):
         result, bins = cut(data, 3, retbins=True)
         tm.assert_series_equal(Series(result), expected)
 
+    @pytest.mark.parametrize('bins', [
+        3, [Timestamp('2013-01-01 04:57:07.200000'),
+            Timestamp('2013-01-01 21:00:00'),
+            Timestamp('2013-01-02 13:00:00'),
+            Timestamp('2013-01-03 05:00:00')]])
+    @pytest.mark.parametrize('box', [list, np.array, Index, Series])
+    def test_datetimetz_cut(self, bins, box):
+        # GH 19872
+        tz = 'US/Eastern'
+        s = Series(date_range('20130101', periods=3, tz=tz))
+        if not isinstance(bins, int):
+            bins = box(bins)
+        result = cut(s, bins)
+        expected = (
+            Series(IntervalIndex([
+                Interval(Timestamp('2012-12-31 23:57:07.200000', tz=tz),
+                         Timestamp('2013-01-01 16:00:00', tz=tz)),
+                Interval(Timestamp('2013-01-01 16:00:00', tz=tz),
+                         Timestamp('2013-01-02 08:00:00', tz=tz)),
+                Interval(Timestamp('2013-01-02 08:00:00', tz=tz),
+                         Timestamp('2013-01-03 00:00:00', tz=tz))]))
+            .astype(CDT(ordered=True)))
+        tm.assert_series_equal(result, expected)
+
+    @pytest.mark.parametrize('bins', [3, np.linspace(0, 1, 4)])
+    def test_datetimetz_qcut(self, bins):
+        # GH 19872
+        tz = 'US/Eastern'
+        s = Series(date_range('20130101', periods=3, tz=tz))
+        result = qcut(s, bins)
+        expected = (
+            Series(IntervalIndex([
+                Interval(Timestamp('2012-12-31 23:59:59.999999999', tz=tz),
+                         Timestamp('2013-01-01 16:00:00', tz=tz)),
+                Interval(Timestamp('2013-01-01 16:00:00', tz=tz),
+                         Timestamp('2013-01-02 08:00:00', tz=tz)),
+                Interval(Timestamp('2013-01-02 08:00:00', tz=tz),
+                         Timestamp('2013-01-03 00:00:00', tz=tz))]))
+            .astype(CDT(ordered=True)))
+        tm.assert_series_equal(result, expected)
+
     def test_datetime_bin(self):
         data = [np.datetime64('2012-12-13'), np.datetime64('2012-12-15')]
         bin_data = ['2012-12-12', '2012-12-14', '2012-12-16']
         expected = (
-            Series(IntervalIndex.from_intervals([
+            Series(IntervalIndex([
                 Interval(Timestamp(bin_data[0]), Timestamp(bin_data[1])),
                 Interval(Timestamp(bin_data[1]), Timestamp(bin_data[2]))]))
-            .astype('category', ordered=True))
+            .astype(CDT(ordered=True)))
 
         for conv in [Timestamp, Timestamp, np.datetime64]:
             bins = [conv(v) for v in bin_data]
@@ -502,13 +600,8 @@ class TestCut(object):
 
         result = cut(date_range('20130102', periods=5),
                      bins=date_range('20130101', periods=2))
-        mask = result.categories.isnull()
+        mask = result.categories.isna()
         tm.assert_numpy_array_equal(mask, np.array([False]))
-        mask = result.isnull()
+        mask = result.isna()
         tm.assert_numpy_array_equal(
             mask, np.array([False, True, True, True, True]))
-
-
-def curpath():
-    pth, _ = os.path.split(os.path.abspath(__file__))
-    return pth
